#!/usr/bin/python3.5
"""pyAgrum setup file
See:
https://pyagrum.lip6.fr
"""

# Always prefer setuptools over distutils
try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension
from io import open
import fnmatch
import glob
import os
import shutil
import subprocess
import sys
from distutils.spawn import find_executable

def build_lrslib():
  """Builds the LRSLIB on system using clang.
  
  This is necessary because distutils default compilator does not enable to set
  different flags for C and C++ files and Clang won't compile C files while
  passing C++ options.
  """
  if os.path.isdir('pyAgrum/lrslib/build'):
    shutil.rmtree('pyAgrum/lrslib/build')
  os.mkdir('pyAgrum/lrslib/build')
  os.chdir('pyAgrum/lrslib/build')
  cmake = find_executable('cmake')
  subprocess.call("cmake ..", shell=True)
  subprocess.call("make", shell=True)
  os.chdir('../../..')

# BEFORE importing setuptools, remove MANIFEST. Otherwise it may not be
# properly updated when the contents of directories change (true for distutils,
# not sure about setuptools).
if os.path.exists('MANIFEST'):
  os.remove('MANIFEST')

def find_files(path, ext):
  """Find all files in path with extension ext."""
  matches = []
  for root, dirnames, filenames in os.walk(path):
      for filename in fnmatch.filter(filenames, ext):
          matches.append(os.path.join(root, filename))
  return matches

#cpp_files = list(glob.iglob('pyAgrum/aGrUM/*/*.cpp'))
cpp_files = find_files('pyAgrum/aGrUM', '*.cpp')
c_files = [ 'pyAgrum/aGrUM/agrum/core/math/lrslib/lrslib.c'
          , 'pyAgrum/aGrUM/agrum/core/math/lrslib/lrsmp.c' ]
includes=['pyAgrum/extensions', 'pyAgrum/aGrUM']
macros=[('NDEBUG', None), ('agrum_EXPORTS', None) ]

cur_version = sys.version_info
if os.path.exists('pyAgrum/pyAgrum.py'):
  os.remove('pyAgrum/pyAgrum.py')
if os.path.exists('pyAgrum/pyAgrumPYTHON_wrap.h'):
  os.remove('pyAgrum/pyAgrumPYTHON_wrap.h')
if os.path.exists('pyAgrum/pyAgrumPYTHON_wrap.cxx'):
  os.remove('pyAgrum/pyAgrumPYTHON_wrap.cxx')
if cur_version[0] == 2:
  shutil.copy('pyAgrum/pyAgrum.py27', 'pyAgrum/pyAgrum.py')
  shutil.copy('pyAgrum/pyAgrumPYTHON2.7_wrap.h', 'pyAgrum/pyAgrumPYTHON_wrap.h')
  shutil.copy('pyAgrum/pyAgrumPYTHON2.7_wrap.cxx', 'pyAgrum/pyAgrumPYTHON_wrap.cxx')
else:
  shutil.copy('pyAgrum/pyAgrum.py35', 'pyAgrum/pyAgrum.py')
  shutil.copy('pyAgrum/pyAgrumPYTHON3.5_wrap.h', 'pyAgrum/pyAgrumPYTHON_wrap.h')
  shutil.copy('pyAgrum/pyAgrumPYTHON3.5_wrap.cxx', 'pyAgrum/pyAgrumPYTHON_wrap.cxx')

flags=[]
link_flags=[]
static_libs=[]
if os.name == 'nt':
  macros.extend([ ('_WINDOWS', None)
                , ('WIN32', None)
                , ('NOMINMAX', None)
                , ('_CRT_SECURE_NO_WARNINGS', None)
                ,	('_CRT_NONSTDC_NO_DEPRECATE', None)
                , ('_MBCS', None)])
  flags=[ '/W3'
        , '/WX-'
        , '/Gm-'
        , '/EHsc'
        , '/MD'
        , '/GS'
        , '/fp:precise'
        , '/Zc:wchar_t'
        , '/Zc:forScope'
        , '/Zc:inline'
        , '/GR'
        , '/openmp'
        , '/Gd'
        , '/TP'
        , '/errorReport:queue'
        , '/bigobj' ]
else:
  if os.environ.get('CXX') != None and 'clang' in os.environ.get('CXX'):
    flags=['-fPIC', '-std=c++14', '-Wall', '-pedantic']
    static_libs =['pyAgrum/lrslib/build/liblrs.a']
    c_files=[]
    includes.append('pyAgrum/lrslib/src')
    macros.append(('__PYAGRUM_MACOS__', None))
    if find_executable('cmake') == None:
      print('Please install cmake to build source using clang.')
      exit(1)
    build_lrslib()
  else:
    flags=['-ffunction-sections'
          , '-fdata-sections'
          , '-Os'
          , '-fPIC'
          , '-fopenmp'
          , '-std=c++14' ]
    link_flags=['-lgomp']

README = ''
with open('README.rst', mode='r', encoding='utf-8') as fd:
  README = fd.read()

my_ext = Extension( name='_pyAgrum',
                    sources=['pyAgrum/pyAgrumPYTHON_wrap.cxx'] + cpp_files + c_files,
                    include_dirs=includes,
                    language='c++',
                    extra_compile_args=flags,
                    define_macros=macros,
                    extra_link_args=static_libs + link_flags
                    #libraries=['gomp']
                  )

setup(
  name='pyagrum',
  version='0.10.1.1',
  description='pyAgrum is a Python wrapper for the C++ aGrUM library',
  url='https://pyagrum.lip6.fr',
  maintainer='Pierre-Henri Wuillemain and Christophe Gonzales',
  maintainer_email='agrum@lip6.fr',
  platforms=['any'],
  license='GPL',
  packages=['pyAgrum', 'pyAgrum/lib', 'pyAgrum/lib/_utils'],
  long_description=README,
  include_package_data=True,
  ext_modules=[my_ext],
  install_requires=['numpy>=1.8.1', 'scipy>=0.13.3', 'matplotlib>=1.3.1', 'pydotplus'],
  classifiers=[
  'Development Status :: 4 - Beta', 
  'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
  'Operating System :: Microsoft :: Windows',
  'Operating System :: MacOS :: MacOS X', 
  'Operating System :: POSIX :: Linux',
  'Programming Language :: C++',
  'Programming Language :: Python',
  'Programming Language :: Python :: 3',
  'Programming Language :: Python :: 3.5',
  'Programming Language :: Python :: 2',
  'Programming Language :: Python :: 2.7',
  'Topic :: Scientific/Engineering :: Artificial Intelligence',
  'Topic :: Scientific/Engineering :: Information Analysis',
  'Topic :: Scientific/Engineering :: Mathematics',
  'Topic :: Software Development :: Libraries',
  'Topic :: Software Development :: Libraries :: Python Modules',
  'Intended Audience :: Developers',
  'Intended Audience :: Financial and Insurance Industry',
  'Intended Audience :: Information Technology',
  'Intended Audience :: Science/Research',
  ],
  keywords='probabilities probabilistic-graphical-models inference diagnosis',
  )

